==> synchro-0.0.5.tgz: package/index.js <==
/**
 * synchro
 * Copyright(c) 2011 Jesus A. Domingo <jesus.domingo@gmail.com>
 * MIT Licensed
 */

/**
 * some dependencies
 */
const util = require('util'),
      events = require('events');

/**
 * the step chainer
 */
function Synchro() {
  this._async = arguments[0] || false;
  this._steps = [];
  this._pending = 0;
}

// for future use (listen on chains?)
util.inherits(Synchro, events.EventEmitter);

// add a function to be executed
Synchro.prototype.add = function (cb) {
  
  var step = new Step(cb),
      that = this;

  step.on('done', function () {
    if (false == that._async) {
      that.run();
    } else {
      that._updatePending();
    }
  });

  step.on('aborted', function () { that.abort(); });

  this._steps.push(step);
  this._pending++;

  return this;
};

// execute the chain
Synchro.prototype.run = function () {
  if (false == this._async) {
    if (this._steps.length) {
      this._steps.shift().run();
    } else {
      this.emit('done');
    }
  } else {
    var o = this._steps.shift();
    while (o) {
      o.run();
      o = this._steps.shift();
    }
    this.emit('started');
  }
};

// checks for pending async tasks
Synchro.prototype._updatePending = function () {
  this._pending--;
  if (!this._pending) {
    this.emit('done');
  }
};

// clean up the chain
Synchro.prototype.abort = function () {
  if (false == this._async) {
    throw new Exception('Asynchronous chains cannot be aborted');
  }
  if (this._steps.length) {
    this._steps.forEach(function (obj) {
      delete obj;
    });
    this._steps.length = 0;
  }
  this.emit('aborted');
};

/**
 * functions to be executed are wrapped as steps
 */
function Step(cb) {
  if ('function' != typeof cb) {
    throw new Exception('Invalid callback');
  }
  this._cb = cb;
}

util.inherits(Step, events.EventEmitter);

// execute this function/step
Step.prototype.run = function () {
  // no scope passed, maybe accept one in the future
  this._cb.call(undefined, this);
};

// emit a 'done' event to the chainer
Step.prototype.next = function () {
  this.emit('done');
};

// emit an abort to the chainer
Step.prototype.abort = function () {
  this.emit('aborted');
};

exports.create = function (async) {
  return new Synchro(async);
};

==> synchro-0.0.6.tgz: package/index.js <==
/**
 * synchro
 * Copyright(c) 2011 Jesus A. Domingo <jesus.domingo@gmail.com>
 * MIT Licensed
 */

/**
 * some dependencies
 */
const util = require('util'),
      events = require('events');

/**
 * the runner chainer
 */
function Synchro() {
  this._async = arguments[0] || false;
  this._runners = [];
  this._pending = 0;
}

// for future use (listen on chains?)
util.inherits(Synchro, events.EventEmitter);

// add a function to be executed
Synchro.prototype.add = function (cb) {
  
  var runner = new Runner(cb),
      that = this;

  runner.on('done', function () {
    if (false == that._async) {
      that.run();
    } else {
      that._updatePending();
    }
  });

  runner.on('aborted', function () { that.abort(); });

  this._runners.push(runner);
  this._pending++;

  return this;
};

// execute the chain
Synchro.prototype.run = function () {
  if (false == this._async) {
    if (this._runners.length) {
      this._runners.shift().run();
    } else {
      this.emit('done');
    }
  } else {
    var o = this._runners.shift();
    while (o) {
      o.run();
      o = this._runners.shift();
    }
    this.emit('started');
  }
};

// checks for pending async tasks
Synchro.prototype._updatePending = function () {
  this._pending--;
  if (!this._pending) {
    this.emit('done');
  }
};

// clean up the chain
Synchro.prototype.abort = function () {
  if (false == this._async) {
    throw new Exception('Asynchronous chains cannot be aborted');
  }
  if (this._runners.length) {
    this._runners.forEach(function (obj) {
      delete obj;
    });
    this._runners.length = 0;
  }
  this.emit('aborted');
};

/**
 * functions to be executed are wrapped as runners
 */
function Runner(cb) {
  if ('function' != typeof cb) {
    throw new Exception('Invalid callback');
  }
  this._cb = cb;
}

util.inherits(Runner, events.EventEmitter);

// execute this function/runner
Runner.prototype.run = function () {
  // no scope passed, maybe accept one in the future
  this._cb.call(undefined, this);
};

// emit a 'done' event to the chainer
Runner.prototype.next = function () {
  this.emit('done');
};

// emit an abort to the chainer
Runner.prototype.abort = function () {
  this.emit('aborted');
};

exports.Synchro = Synchro;
exports.Runner = Runner;

exports.create = function (async) {
  return new Synchro(async);
};
